# # from ezorm.variables import EzORM
# from ezorm.core import EzORM
# from ezorm.config import DATABASE
# from ezorm.crud import *
# from ezorm.db_management import create_tables, delete_tables
# from ezorm.utils import execute
from ezorm.configuration import settings as ezo